/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>

#include "ui_sensorlist.h"

enum
{
  COL_NAME = 0,
  COL_TEMP,
  COL_COLOR,
  COL_ENABLED,
  COL_COLOR_STR,
  COLS_COUNT
} ;

void ui_sensorlist_update(struct ui_sensorlist* list) 
{	
	GtkTreeIter iter;
	GtkTreeModel* model 
		= gtk_tree_view_get_model(GTK_TREE_VIEW(list->widget));
	gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
	struct psensor** sensor = list->sensors;

	while(valid && *sensor) {
		char buf[200];
		sprintf(buf, "%3.0fC", (*sensor)->temps[MAX_TEMPS-1]);

		gtk_list_store_set (GTK_LIST_STORE(model), &iter,
				    COL_TEMP, buf,
				    -1);

		valid = gtk_tree_model_iter_next (model, &iter);
		sensor++;
	}
}

void ui_sensorlist_on_toggled(GtkCellRendererToggle *cell,
			      gchar                 *path_str,
			      gpointer               data)
{
	struct ui_sensorlist* list = (struct ui_sensorlist*)data;
	GtkTreeModel* model = gtk_tree_view_get_model(GTK_TREE_VIEW(list->widget));
	GtkTreeIter  iter;
	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
	gboolean fixed;	

	gtk_tree_model_get_iter (model, &iter, path);
	gtk_tree_model_get (model, &iter, COL_ENABLED, &fixed, -1);

	fixed ^= 1;

	gint* i = gtk_tree_path_get_indices(path);
	if (i) {
		int n = *i;
		struct psensor** sensor = list->sensors;
		while(n--) 
			sensor++;		
		(*sensor)->enabled = fixed;
	}

	gtk_list_store_set (GTK_LIST_STORE (model), &iter, COL_ENABLED, fixed, -1);
	
	gtk_tree_path_free (path);
} 

struct ui_sensorlist* ui_sensorlist_create(struct psensor** sensors)
{
	struct ui_sensorlist* list
		= malloc(sizeof(struct ui_sensorlist));

	list->sensors = sensors;       

	GtkListStore *store = gtk_list_store_new (COLS_COUNT,
						  G_TYPE_STRING, 
						  G_TYPE_STRING, 
						  G_TYPE_STRING,
						  G_TYPE_BOOLEAN, 
						  G_TYPE_STRING);
	GtkWidget* view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(store));

	list->widget = view;

	GtkCellRenderer* renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
						     -1,      
						     "Sensor",  
						     renderer,
						     "text", COL_NAME,
						     NULL);

	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
						     -1,      
						     "Temp", 
						     renderer,
						     "text", COL_TEMP,
						     NULL);

	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
						     -1,      
						     "Color", 
						     renderer,
						     "text", COL_COLOR,
						     "background",COL_COLOR_STR,
						     NULL);

	renderer = gtk_cell_renderer_toggle_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
						     -1,      
						     "Enabled",  
						     renderer,
						     "active",COL_ENABLED,
						     NULL);
	g_signal_connect (G_OBJECT(renderer), "toggled",
			  G_CALLBACK(ui_sensorlist_on_toggled), list);

	struct psensor** sensor_cur = sensors;
	while(*sensor_cur) {
		GtkTreeIter iter;
		GdkColor color;
		color.red = (*sensor_cur)->color[0] * 65535;
		color.green = (*sensor_cur)->color[1] * 65535;
		color.blue = (*sensor_cur)->color[2] * 65535;

		gchar* scolor = gdk_color_to_string(&color);

		gtk_list_store_append (store, &iter);
		gtk_list_store_set (store, &iter,
				    COL_NAME, (*sensor_cur)->name,
				    COL_TEMP, "N/A",
				    COL_ENABLED, (*sensor_cur)->enabled,
				    COL_COLOR_STR, scolor,
				    -1);

		free(scolor);

		sensor_cur++;
	}

	return list;

}
